@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2020, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -42,16 +42,15 @@
 #ifndef MYSQL_ABI_CHECK
 #include <stdbool.h>
 #include <stddef.h>
+#include <stdint.h>
 #include <sys/types.h>
 #endif
 
-// Small extra definitions to avoid pulling in my_inttypes.h in client code.
-// IWYU pragma: no_include "my_inttypes.h"
-#if defined(_WIN32)
-typedef unsigned __int64 my_ulonglong;
-#else
-typedef unsigned long long my_ulonglong;
-#endif /* _WIN32 */
+// Legacy definition for the benefit of old code. Use uint64_t in new code.
+// If you get warnings from printf, use the PRIu64 macro, or, if you need
+// compatibility with older versions of the client library, cast
+// before printing.
+typedef uint64_t my_ulonglong;
 
 #ifndef my_socket_defined
 #define my_socket_defined
@@ -76,7 +75,7 @@ typedef int my_socket;
 #endif
 #endif /* MY_COMPILER_INCLUDED */
 
-#include "binary_log_types.h"
+#include "field_types.h"
 #include "my_list.h"
 #include "mysql_com.h"
 
@@ -145,7 +144,7 @@ typedef struct MYSQL_FIELD {
 typedef char **MYSQL_ROW;                /* return data as array of strings */
 typedef unsigned int MYSQL_FIELD_OFFSET; /* offset to current field */
 
-#define MYSQL_COUNT_ERROR (~(my_ulonglong)0)
+#define MYSQL_COUNT_ERROR (~(uint64_t)0)
 
 /* backward compatibility define - to be removed eventually */
 #define ER_WARN_DATA_TRUNCATED WARN_DATA_TRUNCATED
@@ -163,7 +162,7 @@ struct MEM_ROOT;
 typedef struct MYSQL_DATA {
   MYSQL_ROWS *data;
   struct MEM_ROOT *alloc;
-  my_ulonglong rows;
+  uint64_t rows;
   unsigned int fields;
 } MYSQL_DATA;
 
@@ -207,7 +206,11 @@ enum mysql_option {
   MYSQL_OPT_GET_SERVER_PUBLIC_KEY,
   MYSQL_OPT_RETRY_COUNT,
   MYSQL_OPT_OPTIONAL_RESULTSET_METADATA,
-  MYSQL_OPT_SSL_FIPS_MODE
+  MYSQL_OPT_SSL_FIPS_MODE,
+  MYSQL_OPT_TLS_CIPHERSUITES,
+  MYSQL_OPT_COMPRESSION_ALGORITHMS,
+  MYSQL_OPT_ZSTD_COMPRESSION_LEVEL,
+  MYSQL_OPT_LOAD_DATA_LOCAL_DIR
 };
 
 /**
@@ -298,9 +301,9 @@ typedef struct MYSQL {
   struct CHARSET_INFO *charset;
   MYSQL_FIELD *fields;
   struct MEM_ROOT *field_alloc;
-  my_ulonglong affected_rows;
-  my_ulonglong insert_id;  /* id if insert on table with NEXTNR */
-  my_ulonglong extra_info; /* Not used */
+  uint64_t affected_rows;
+  uint64_t insert_id;      /* id if insert on table with NEXTNR */
+  uint64_t extra_info;     /* Not used */
   unsigned long thread_id; /* Id for connection in server */
   unsigned long packet_length;
   unsigned int port;
@@ -331,7 +334,7 @@ typedef struct MYSQL {
 } MYSQL;
 
 typedef struct MYSQL_RES {
-  my_ulonglong row_count;
+  uint64_t row_count;
   MYSQL_FIELD *fields;
   struct MYSQL_DATA *data;
   MYSQL_ROWS *data_cursor;
@@ -367,13 +370,13 @@ typedef struct MYSQL_RES {
   @sa mysql_binlog_close()
 */
 typedef struct MYSQL_RPL {
-  size_t file_name_length;     /** Length of the 'file_name' or 0     */
-  const char *file_name;       /** Filename of the binary log to read */
-  my_ulonglong start_position; /** Position in the binary log to      */
-                               /*  start reading from                 */
-  unsigned int server_id;      /** Server ID to use when identifying  */
-                               /*  with the master                    */
-  unsigned int flags;          /** Flags, e.g. MYSQL_RPL_GTID         */
+  size_t file_name_length; /** Length of the 'file_name' or 0     */
+  const char *file_name;   /** Filename of the binary log to read */
+  uint64_t start_position; /** Position in the binary log to      */
+                           /*  start reading from                 */
+  unsigned int server_id;  /** Server ID to use when identifying  */
+                           /*  with the master                    */
+  unsigned int flags;      /** Flags, e.g. MYSQL_RPL_GTID         */
 
   /** Size of gtid set data              */
   size_t gtid_set_encoded_size;
@@ -424,7 +427,7 @@ void STDCALL mysql_thread_end(void);
   Should definitely be used if one uses shared libraries.
 */
 
-my_ulonglong STDCALL mysql_num_rows(MYSQL_RES *res);
+uint64_t STDCALL mysql_num_rows(MYSQL_RES *res);
 unsigned int STDCALL mysql_num_fields(MYSQL_RES *res);
 bool STDCALL mysql_eof(MYSQL_RES *res);
 MYSQL_FIELD *STDCALL mysql_fetch_field_direct(MYSQL_RES *res,
@@ -435,8 +438,8 @@ MYSQL_FIELD_OFFSET STDCALL mysql_field_tell(MYSQL_RES *res);
 enum enum_resultset_metadata STDCALL mysql_result_metadata(MYSQL_RES *result);
 
 unsigned int STDCALL mysql_field_count(MYSQL *mysql);
-my_ulonglong STDCALL mysql_affected_rows(MYSQL *mysql);
-my_ulonglong STDCALL mysql_insert_id(MYSQL *mysql);
+uint64_t STDCALL mysql_affected_rows(MYSQL *mysql);
+uint64_t STDCALL mysql_insert_id(MYSQL *mysql);
 unsigned int STDCALL mysql_errno(MYSQL *mysql);
 const char *STDCALL mysql_error(MYSQL *mysql);
 const char *STDCALL mysql_sqlstate(MYSQL *mysql);
@@ -465,6 +468,20 @@ int STDCALL mysql_real_query(MYSQL *mysql, const char *q, unsigned long length);
 MYSQL_RES *STDCALL mysql_store_result(MYSQL *mysql);
 MYSQL_RES *STDCALL mysql_use_result(MYSQL *mysql);
 
+enum net_async_status STDCALL mysql_real_connect_nonblocking(
+    MYSQL *mysql, const char *host, const char *user, const char *passwd,
+    const char *db, unsigned int port, const char *unix_socket,
+    unsigned long clientflag);
+enum net_async_status STDCALL mysql_send_query_nonblocking(
+    MYSQL *mysql, const char *query, unsigned long length);
+enum net_async_status STDCALL mysql_real_query_nonblocking(
+    MYSQL *mysql, const char *query, unsigned long length);
+enum net_async_status STDCALL
+mysql_store_result_nonblocking(MYSQL *mysql, MYSQL_RES **result);
+enum net_async_status STDCALL mysql_next_result_nonblocking(MYSQL *mysql);
+enum net_async_status STDCALL mysql_select_db_nonblocking(MYSQL *mysql,
+                                                          const char *db,
+                                                          bool *error);
 void STDCALL mysql_get_character_set_info(MYSQL *mysql,
                                           MY_CHARSET_INFO *charset);
 
@@ -510,12 +527,16 @@ int STDCALL mysql_options4(MYSQL *mysql, enum mysql_option option,
 int STDCALL mysql_get_option(MYSQL *mysql, enum mysql_option option,
                              const void *arg);
 void STDCALL mysql_free_result(MYSQL_RES *result);
-void STDCALL mysql_data_seek(MYSQL_RES *result, my_ulonglong offset);
+enum net_async_status STDCALL mysql_free_result_nonblocking(MYSQL_RES *result);
+void STDCALL mysql_data_seek(MYSQL_RES *result, uint64_t offset);
 MYSQL_ROW_OFFSET STDCALL mysql_row_seek(MYSQL_RES *result,
                                         MYSQL_ROW_OFFSET offset);
 MYSQL_FIELD_OFFSET STDCALL mysql_field_seek(MYSQL_RES *result,
                                             MYSQL_FIELD_OFFSET offset);
 MYSQL_ROW STDCALL mysql_fetch_row(MYSQL_RES *result);
+enum net_async_status STDCALL mysql_fetch_row_nonblocking(MYSQL_RES *res,
+                                                          MYSQL_ROW *row);
+
 unsigned long *STDCALL mysql_fetch_lengths(MYSQL_RES *result);
 MYSQL_FIELD *STDCALL mysql_fetch_field(MYSQL_RES *result);
 MYSQL_RES *STDCALL mysql_list_fields(MYSQL *mysql, const char *table,
@@ -659,8 +680,8 @@ typedef struct MYSQL_STMT {
   */
   int (*read_row_func)(struct MYSQL_STMT *stmt, unsigned char **row);
   /* copy of mysql->affected_rows after statement execution */
-  my_ulonglong affected_rows;
-  my_ulonglong insert_id;      /* copy of mysql->insert_id */
+  uint64_t affected_rows;
+  uint64_t insert_id;          /* copy of mysql->insert_id */
   unsigned long stmt_id;       /* Id for prepared statement */
   unsigned long flags;         /* i.e. type of cursor to open */
   unsigned long prefetch_rows; /* number of rows per one COM_FETCH */
@@ -710,6 +731,9 @@ enum enum_stmt_attr_type {
   STMT_ATTR_PREFETCH_ROWS
 };
 
+bool STDCALL mysql_bind_param(MYSQL *mysql, unsigned n_params,
+                              MYSQL_BIND *binds, const char **names);
+
 MYSQL_STMT *STDCALL mysql_stmt_init(MYSQL *mysql);
 int STDCALL mysql_stmt_prepare(MYSQL_STMT *stmt, const char *query,
                                unsigned long length);
@@ -741,10 +765,10 @@ const char *STDCALL mysql_stmt_sqlstate(MYSQL_STMT *stmt);
 MYSQL_ROW_OFFSET STDCALL mysql_stmt_row_seek(MYSQL_STMT *stmt,
                                              MYSQL_ROW_OFFSET offset);
 MYSQL_ROW_OFFSET STDCALL mysql_stmt_row_tell(MYSQL_STMT *stmt);
-void STDCALL mysql_stmt_data_seek(MYSQL_STMT *stmt, my_ulonglong offset);
-my_ulonglong STDCALL mysql_stmt_num_rows(MYSQL_STMT *stmt);
-my_ulonglong STDCALL mysql_stmt_affected_rows(MYSQL_STMT *stmt);
-my_ulonglong STDCALL mysql_stmt_insert_id(MYSQL_STMT *stmt);
+void STDCALL mysql_stmt_data_seek(MYSQL_STMT *stmt, uint64_t offset);
+uint64_t STDCALL mysql_stmt_num_rows(MYSQL_STMT *stmt);
+uint64_t STDCALL mysql_stmt_affected_rows(MYSQL_STMT *stmt);
+uint64_t STDCALL mysql_stmt_insert_id(MYSQL_STMT *stmt);
 unsigned int STDCALL mysql_stmt_field_count(MYSQL_STMT *stmt);
 
 bool STDCALL mysql_commit(MYSQL *mysql);
@@ -766,6 +790,12 @@ void STDCALL mysql_reset_server_public_key(void);
 
 #define HAVE_MYSQL_REAL_CONNECT
 
+MYSQL *STDCALL mysql_real_connect_dns_srv(MYSQL *mysql,
+                                          const char *dns_srv_name,
+                                          const char *user, const char *passwd,
+                                          const char *db,
+                                          unsigned long client_flag);
+
 #ifdef __cplusplus
 }
 #endif
